@@ -30,7 +30,7 @@ from thermo.uniquac import UNIQUAC
 from random import random
 from thermo import *
 import numpy as np
-from fluids.numerics import jacobian, hessian, derivative, normalize, assert_close, assert_close1d, assert_close2d, assert_close3d
+from fluids.numerics import jacobian, hessian, derivative, normalize, assert_close, assert_close1d, assert_close2d, assert_close3d, linspace
 from thermo.test_utils import check_np_output_activity
 import pickle
 
@@ -118,6 +118,9 @@ def test_UNIQUAC_no_coefficients():
     rs = [2.1055, 0.9200]
     qs = [1.972, 1.400]
     GE = UNIQUAC(T=T, xs=xs, rs=rs, qs=qs, ABCDEF=tuple())
+    
+    # Gammas ARE NOT equal to 1!
+    # assert_close1d(GE.gammas(), [1, 1], rtol=1e-13)
 
 def test_UNIQUAC_madeup_ternary():
     N = 3
@@ -141,6 +144,8 @@ def test_UNIQUAC_madeup_ternary():
          UNIQUAC(T=T, xs=xs, rs=rs, qs=qs, ABCDEF=(tausA, None, None, []))
     with pytest.raises(ValueError):
          UNIQUAC(T=T, xs=xs, rs=rs, qs=qs, ABCDEF=(tausA, None, None, [1, 1]))
+    with pytest.raises(ValueError):
+         UNIQUAC(T=T, xs=xs, rs=rs, qs=qs, ABCDEF=(tausA, None, None, [1, 1, 1]))
     with pytest.raises(ValueError):
          UNIQUAC(T=T, xs=xs, rs=rs, qs=qs, ABCDEF=(tausA, None, None, [[0.0, 9.64e-8], [1.53e-7, 0.0, 1.11e-7], [7.9e-8, 2.276e-8, 0]]))
     with pytest.raises(ValueError):
@@ -254,12 +259,13 @@ def test_UNIQUAC_madeup_ternary():
     GE2 = UNIQUAC.from_json(GE.as_json())
     assert GE2.__dict__ == GE.__dict__
 
-def test_UNIQUAC_require_parameters():
+def test_UNIQUAC_no_parameters():
     xs = [0.7273, 0.0909, 0.1818]
     rs = [.92, 2.1055, 3.1878]
     qs = [1.4, 1.972, 2.4]
-    with pytest.raises(ValueError):
-        GE = UNIQUAC(T=300.0, xs=xs, rs=rs, qs=qs)
+    
+    # Bad idea but it's a thing you actually do
+    GE = UNIQUAC(T=300.0, xs=xs, rs=rs, qs=qs)
 
 def test_UNIQUAC_numpy_inputs():
 
@@ -352,4 +358,47 @@ def test_Uniquac_numpy_output_correct_array_internal_ownership():
                  'tau_coeffs_D', 'tau_coeffs_E', 'tau_coeffs_F'):
         obj = getattr(modelnp, name)
         assert obj.flags.c_contiguous
-        assert obj.flags.owndata
\ No newline at end of file
+        assert obj.flags.owndata
+        
+@pytest.mark.fitting
+def test_UNIQUAC_fitting_gamma_1_success_cases():
+    pts = 10
+    rs = [3.8254, 4.4998]
+    qs = [3.316, 3.856]
+    xs = [[xi, 1.0 - xi] for xi in linspace(1e-7, 1-1e-7, pts)]
+    gammas = [[1, 1] for i in range(pts)]
+    coeffs, stats = UNIQUAC.regress_binary_parameters(gammas, xs, rs, qs)
+    assert stats['MAE'] < 1e-5
+    
+    
+def test_UNIQUAC_chemsep():
+    from thermo.interaction_parameters import IPDB
+    xs = [0.252, 0.748]
+    rs = [2.11, 0.92]
+    qs = [1.97, 1.400]
+    N = 2
+    T = 343.15
+    tausA = tausC = tausD = tausE = tausF = [[0.0]*N for _ in range(N)]
+    gammas_expect = [1.977454791958557, 1.1397696289861017]
+    tausB = IPDB.get_ip_asymmetric_matrix(name='ChemSep UNIQUAC', CASs=['64-17-5', '7732-18-5'], ip='bij')
+    ABCDEF = (tausA, tausB, tausC, tausD, tausE, tausF)
+    GE = UNIQUAC(T=T, xs=xs, rs=rs, qs=qs, ABCDEF=ABCDEF)
+    gammas = GE.gammas()
+    # Checked against ChemSep TPxy plot/data table - shows 5 decimal places
+    assert_close1d(gammas, gammas_expect, rtol=1e-12)
+    
+    # Partial inputs
+    GE = UNIQUAC(T=T, xs=xs, rs=rs, qs=qs, tau_bs=tausB)
+    assert_close1d(GE.gammas(), gammas_expect, rtol=1e-12)
+
+
+
+def test_UNIQUAC_one_component():
+    GE = UNIQUAC(T=300, xs=[1], rs=[1], qs=[1], ABCDEF=([[0.0]], [[0.0]], [[0.0]], [[0.0]], [[0.0]], [[0.0]]))
+    
+    # d2GE_dTdxs, d2GE_dxixjs, dGE_dxs, dSE_dxs all do not have to be zero because the equation can be evaluated with x != 0.
+    
+    for s in GE._point_properties:
+        if hasattr(GE, s):
+            res = getattr(GE, s)()
+            # print(res, s)
