@@ -26,7 +26,7 @@ __all__ = ['alpha_Twu91_objf', 'alpha_Twu91_objfc', 'fit_function',
            'Twu91_check_params', 'postproc_lmfit',
            'alpha_poly_objf', 'alpha_poly_objfc', 'poly_check_params',
            'fit_cheb_poly', 'poly_fit_statistics', 'fit_cheb_poly_auto',
-           'data_fit_statistics']
+           'data_fit_statistics', 'fit_customized']
 
 from fluids.numerics import (chebval, brenth, third, sixth, roots_cubic,
                              roots_cubic_a1, numpy as np, newton,
@@ -36,14 +36,21 @@ from fluids.numerics import (chebval, brenth, third, sixth, roots_cubic,
                              is_poly_positive, is_poly_negative,
                              max_abs_error, max_abs_rel_error, max_squared_error, 
                              max_squared_rel_error, mean_abs_error, mean_abs_rel_error, 
-                             mean_squared_error, mean_squared_rel_error)
+                             mean_squared_error, mean_squared_rel_error,
+                             curve_fit, differential_evolution, fit_minimization_targets, leastsq,
+                             lmder, lmfit)
 from fluids.constants import R
+import fluids, thermo
 try:
     from numpy.polynomial.chebyshev import poly2cheb
     from numpy.polynomial.chebyshev import cheb2poly
     from numpy.polynomial.polynomial import Polynomial
 except:
     pass
+try:
+    from random import uniform
+except:
+    pass
 
 ChebTools = None
 def fit_cheb_poly(func, low, high, n,
@@ -533,4 +540,176 @@ def fit_function(fun, x0=None, args=None, check_fun=None, debug=False,
                         print(result.aard, method)
                     else:
                         print(result.aard, check_fun(fit), method)
-    return best_coeffs, best_result
\ No newline at end of file
+    return best_coeffs, best_result
+
+
+
+
+def fit_customized(Ts, data, fitting_func, fit_parameters, use_fit_parameters, 
+                   fit_method, objective, multiple_tries_max_objective, 
+                   guesses=None, initial_guesses=None, analytical_jac=None,
+                   solver_kwargs=None, use_numba=False, multiple_tries=False,
+                   do_statistics=False, multiple_tries_max_err=1e-5,
+                   func_wrapped_for_leastsq=None, jac_wrapped_for_leastsq=None):
+    if solver_kwargs is None: solver_kwargs = {}
+    if use_numba:
+        fit_func_dict = fluids.numba.numerics.fit_minimization_targets
+    else:
+        fit_func_dict = fit_minimization_targets
+
+    err_func = fit_func_dict[objective]
+    err_fun_multiple_guesses = fit_func_dict[multiple_tries_max_objective]
+    do_minimization = fit_method == 'differential_evolution'
+    
+    if do_minimization:
+        def minimize_func(params):
+            calc = fitting_func(Ts, *params)
+            err = err_func(data, calc)
+            return err
+        
+    p0 = [1.0]*len(fit_parameters)
+    if guesses:
+        for i, k in enumerate(use_fit_parameters):
+            if k in guesses:
+                p0[i] = guesses[k]
+                
+    if initial_guesses:
+        # iterate over all the initial guess parameters we have and find the one
+        # with the lowest error (according to the error criteria)
+        best_hardcoded_guess = None
+        best_hardcoded_err = 1e308
+        hardcoded_errors = []
+        hardcoded_guesses = initial_guesses
+        extra_user_guess = [{k: v for k, v in zip(use_fit_parameters, p0)}]
+        all_iter_guesses = hardcoded_guesses + extra_user_guess
+        array_init_guesses = []
+        err_func_init = fit_func_dict['MeanRelErr']
+        for hardcoded in all_iter_guesses:
+            ph = [None]*len(fit_parameters)
+            for i, k in enumerate(use_fit_parameters):
+                ph[i] = hardcoded[k]
+            array_init_guesses.append(ph)
+            
+            calc = fitting_func(Ts, *ph)
+            err = err_func_init(data, calc)
+            hardcoded_errors.append(err)
+            if err < best_hardcoded_err:
+                best_hardcoded_err = err
+                best_hardcoded_guess = ph
+        p0 = best_hardcoded_guess
+        if best_hardcoded_err == 1e308 and fit_method != 'differential_evolution':
+            raise ValueError("No attemped fitting parameters yielded remotely reasonable errors. Check input data or provide guesses")
+        array_init_guesses = [p0 for _, p0 in sorted(zip(hardcoded_errors, array_init_guesses))]
+    else:
+        array_init_guesses = [p0]
+    
+
+    if func_wrapped_for_leastsq is None:
+        def func_wrapped_for_leastsq(params):
+            # jacobian is the same
+            return fitting_func(Ts, *params) - data
+
+    if jac_wrapped_for_leastsq is None:
+        def jac_wrapped_for_leastsq(params):
+            return analytical_jac(Ts, *params)
+
+    pcov = None
+    if fit_method == 'differential_evolution':
+        if 'bounds' in solver_kwargs:
+            working_bounds = solver_kwargs.pop('bounds')
+        else:
+            factor = 4.0
+            if len(array_init_guesses) > 3:
+                lowers_guess, uppers_guess = np.array(array_init_guesses).min(axis=0), np.array(array_init_guesses).max(axis=0)
+                working_bounds = [(lowers_guess[i]*factor if lowers_guess[i] < 0. else lowers_guess[i]*(1.0/factor),
+                                    uppers_guess[i]*(1.0/factor) if uppers_guess[i] < 0. else uppers_guess[i]*(factor),
+                                    ) for i in range(len(use_fit_parameters))]
+            else:
+                working_bounds = [(0, 1000) for k in use_fit_parameters]
+        popsize = solver_kwargs.get('popsize', 15)*len(fit_parameters)
+        init = array_init_guesses
+        for i in range(len(init), popsize):
+            to_add = [uniform(ll, lh) for ll, lh in working_bounds]
+            init.append(to_add)
+            
+        res = differential_evolution(minimize_func,# init=np.array(init),
+                                      bounds=working_bounds, **solver_kwargs)
+        popt = res['x']
+    else:
+        lm_direct = fit_method == 'lm'
+        Dfun = jac_wrapped_for_leastsq if analytical_jac is not None else None
+        if 'maxfev' not in solver_kwargs and fit_method == 'lm':
+            # DO NOT INCREASE THIS! Make an analytical jacobian instead please.
+            # Fought very hard to bring the analytical jacobian maxiters down to 500!
+            # 250 seems too small.
+            if analytical_jac is not None:
+                solver_kwargs['maxfev'] = 500
+            else:
+                solver_kwargs['maxfev'] = 5000 
+        if multiple_tries:
+            multiple_tries_best_error = 1e300
+            best_popt, best_pcov = None, None
+            popt = None
+            if type(multiple_tries) is int and len(array_init_guesses) > multiple_tries:
+                array_init_guesses = array_init_guesses[0:multiple_tries]
+            for p0 in array_init_guesses:
+                try:
+                    if lm_direct:
+                        if Dfun is not None:
+                            popt = lmder(func_wrapped_for_leastsq, Dfun, p0, tuple(), False,
+                                      0, 1.49012e-8, 1.49012e-8, 0.0, solver_kwargs['maxfev'],
+                                      100, None)[0]
+                        else:
+                            popt, _ = leastsq(func_wrapped_for_leastsq, p0, Dfun=Dfun, **solver_kwargs)
+
+                        pcov = None
+                    else:
+                        popt, pcov = curve_fit(fitting_func, Ts, data, p0=p0, jac=analytical_jac, 
+                                                method=fit_method, **solver_kwargs)
+                except:
+                    continue
+                calc = fitting_func(Ts, *popt)
+                curr_err = err_fun_multiple_guesses(data, calc)
+                if curr_err < multiple_tries_best_error:
+                    best_popt, best_pcov = popt, pcov
+                    multiple_tries_best_error = curr_err
+                    if curr_err < multiple_tries_max_err:
+                        break
+                
+            if best_popt is None:
+                raise ValueError("No guesses converged")
+            else:
+                popt, pcov = best_popt, best_pcov
+        else:
+            if lm_direct:
+                popt, _ = leastsq(func_wrapped_for_leastsq, p0, Dfun=Dfun, **solver_kwargs)
+                pcov = None
+            else:
+                popt, pcov = curve_fit(fitting_func, Ts, data, p0=p0, jac=analytical_jac,
+                                        method=fit_method, **solver_kwargs)
+    out_kwargs = {}
+    for param_name, param_value in zip(fit_parameters, popt):
+        out_kwargs[param_name] = float(param_value)
+
+    if do_statistics:
+        if not use_numba:
+            stats_func = data_fit_statistics 
+        else:
+            stats_func = thermo.numba.fitting.data_fit_statistics
+        calc = fitting_func(Ts, *popt)
+        stats = stats_func(Ts, data, calc)
+        statistics = {}
+        statistics['calc'] = calc
+        statistics['MAE'] = stats[0]
+        statistics['STDEV'] = stats[1]
+        statistics['min_ratio'] = stats[2]
+        statistics['max_ratio'] = stats[3]
+        statistics['pcov'] = pcov
+        return out_kwargs, statistics
+
+
+    return out_kwargs
+
+
+
+
